﻿using MotorPlantContracts.BindingModels;
using MotorPlantContracts.BusinessLogicsContracts;
using MotorPlantContracts.SearchModels;
using MotorPlantContracts.StoragesContracts;
using MotorPlantContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System.Text.RegularExpressions;



namespace MotorPlantBusinessLogic.BusinessLogic
{
	public class ClientLogic : IClientLogic
	{
		private readonly ILogger _logger;
		private readonly IClientStorage _clientStorage;
		public ClientLogic(ILogger<ClientLogic> logger, IClientStorage clientStorage)
		{
			_logger = logger;
			_clientStorage = clientStorage;
		}
		public bool Create(ClientBindingModel model)
		{
			CheckUser(model);
			if (_clientStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(ClientBindingModel model)
		{
			CheckUser(model);
			if (_clientStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(ClientBindingModel model)
		{
			CheckUser(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_clientStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}

		public ClientViewModel? ReadElement(ClientSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. UserName:{UserName}.Id:{Id}", model.Email, model.Id);
			var element = _clientStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}

		public List<ClientViewModel>? ReadList(ClientSearchModel? model)
		{
			_logger.LogInformation("ReadList. Email:{Email}.Id:{ Id} ", model?.Email, model?.Id);
			var list = (model == null) ? _clientStorage.GetFullList() : _clientStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}

		public void CheckUser(ClientBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.ClientFIO))
			{
				throw new ArgumentNullException("Нет ФИО пользователя", nameof(model.ClientFIO));
			}
			if (string.IsNullOrEmpty(model.Email) || !Regex.IsMatch(model.Email, @"^[A-Za-z0-9._%+-]+\@([a-z0-9-]+\.)+[a-z]{2,4}$"))
			{
				throw new ArgumentNullException("Нет указана валидная почта", nameof(model.Email));
			}
			if (!Regex.IsMatch(model.Email, @"^[\w-\.]+@([\w-]+\.)+[\w-]{2,4}$"))
			{
				throw new ArgumentNullException("Не указан правильный пароль", nameof(model.Password));
			}
			_logger.LogInformation("Client. ClientFIO:{ClientFIO}.Email:{Email}.Id:{Id}",
			model.ClientFIO, model.Email, model.Id);
			var element = _clientStorage.GetElement(new ClientSearchModel
			{
				ClientFIO = model.ClientFIO
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Клиент с таким именем уже есть");
			}
		}
	}
}